#! /usr/bin/env python

# name: m2n.py
# version: 0.22
# date: June, 29 2003
# author: Jean-Pierre Sutto (jpsn at free.fr)
# last version: http://jpsn.free.fr/usenet/m2n.html

# english

# This python script is a primitive mail2news for leafnode (very very
# primitive: I wrote it in two evenings, so...). It transforms and
# transports a mailbox in /var/spool/news/local/ and does the right
# things in order to read it with a newsreader.
# 
# usage: m2n.py [-d] mailbox
# 
# We suppose that the script and files have the right permissions. As
# a precaution, the script leaves a backup of the mailbox in /tmp.
# You have to repeat the script as many as mailboxes are.
# 
# Typically, it is made to be used after the distribution of the mails
# and theirs sorting with procmail:
# 
# 1. news receives messages of a mailing list (eg. the list
# tartempion);
# 
# 2. procmail runs and delivers the messages in the right mailboxes
# (eg. in /var/spool/news/mail/tartempion);
# 
# 3. news runs the script with the path to the mailbox (eg. m2n.py
# /var/spool/news/mail/tartempion).
#
# 4. The group will have name: local.<name_of_the_mailbox> (eg.
# local.tartempion) and you will have to run fetch -f the first time you
# create a new local group.
#
# Feel free to change all that you want in this script. As far as I know,
# version 2 of leafnode will support natively local groups.

# francais

# Ce script python est un mail2news primitif pour leafnode (tres tres
# primitif: je l'ai ecrit en deux soirees). Il transforme et transporte
# une mailbox dans /var/spool/news/local/ et fait ce qu'il faut pour
# qu'on puisse lire les messages avec un lecteur de nouvelles.
# 
# utilisation: m2n.py [-d] mailbox
#
# On suppose que le script et les fichiers ont les bons droits. La
# mailbox est par precaution sauvegardee dans /tmp. Il faut repeter
# le script autant de fois qu'il y a de mailbox.
#
# Typiquement, il est concu pour s'utiliser apres la distribution du
# courrier et son tri par procmail:
# 
# 1. news recoit les messages de listes de diffusions (ex: la liste
# tartempion);
# 
# 2. procmail tourne sur le systeme et distribue les messages dans
# les bonnes mailbox (ex: les messages de la liste tartempion dans
# /var/spool/news/mail/tartempion);
# 
# 3. news fait tourner ce script en indiquant la mailbox (ex:
# m2n.py /var/spool/news/mail/tartempion).
#
# 4. Le nouveau groupe aura pour nom: local.<nom_de_la_mailbox> (ex.
# local.tartempion). Il faudra lancer fetch -f aprs chaque cration de
# groupe local.
#
# N'hesitez pas a changer tout ce que voulez dans ce script. Je crois
# que la version 2 de leafnode supportera d'origine les groupes locaux.

#---------------------------------------------------------------#
#                         Routines                              #
#---------------------------------------------------------------#
def errorcommandline():

	import sys
	print """
	usage: m2n.py [-d] mailbox
	"""

	sys.exit(2)

#---------------------------------------------------------------#
def readfile(file):

	f=open(file,'r')
	s=f.read()
	f.close()

	return s

#---------------------------------------------------------------#
def savefile(s,file):

	f=open(file,'w')
	f.write(s)
	f.close()

	return

#---------------------------------------------------------------#
def delhtml(s):

	boundary='''Content-Type: multipart/alternative;
 boundary="'''
	debutboundary=string.find(s,boundary)
	if debutboundary==-1:
		return s	

	finboundary=string.find(s,'"',debutboundary+len(boundary))
	if finboundary==-1:
		return s
	ref=s[debutboundary+len(boundary):finboundary]

	refhtml=ref+'\n'+'Content-Type: text/html'
	debuthtml=string.find(s,refhtml)
	if debuthtml==-1:
		return s

	finhtml=string.find(s,ref,debuthtml+len(refhtml))
	if finhtml==-1:
		return s

	s=s[:debuthtml]+s[finhtml:]
	return s


#---------------------------------------------------------------#
#                            Script                             #
#---------------------------------------------------------------#

import sys
import os
import string
import re
import whrandom
import shutil

tr='--------------------------------------------------\n'
pathnews='/var/spool/news/'
pathids='/var/spool/news/message.id/001/'

# Hack: if you want to try to use the In-Reply-To field
# for references, change the following in UseInReplyTo=1.
# If not, use UseInReplyTo=0
UseInReplyTo=1

# Hack 2: if you want to delete the html part of multiparts messages,
# change the following in DelHtml=1. If not, use DelHtml=0
DelHtml=1

#---------------------------------------------------------------#
# print 'we take the args of the command line'
#---------------------------------------------------------------#

arguments=sys.argv[1:]

if arguments[0:]:
	if arguments[0]=='-d':
		debug=1
		if arguments[1:]:
			pathcompletmailbox=arguments[1]
		else:
			errorcommandline()
	else:
		debug=0
		pathcompletmailbox=arguments[0]
else:
	errorcommandline()

#---------------------------------------------------------------#
if debug: print tr*2+'we check path and file'
#---------------------------------------------------------------#

mailbox=os.path.split(pathcompletmailbox)[1]

if debug: print 'mailbox=',mailbox

directorymailbox=os.path.split(pathcompletmailbox)[0]+'/'
if directorymailbox=='/': 
	directorymailbox='./'

if debug: print 'directorymailbox=',directorymailbox

mailboxexists=os.path.isfile(pathcompletmailbox)

if debug: print "mailboxexists=",mailboxexists

if mailboxexists!=1:
	print "no mailbox "+mailbox+' !'
	sys.exit(2)

#---------------------------------------------------------------#
if debug: print tr+'we go in /tmp/'
#---------------------------------------------------------------#

#command='cp -f '+pathcompletmailbox+' /tmp/'
#if debug: print command
#os.system(command)
#XXX
if debug: print 'cp '+pathcompletmailbox+' /tmp/'
shutil.copy(pathcompletmailbox,'/tmp/') 

os.chdir('/tmp/')

if debug: print 'change of directory: ok'

#---------------------------------------------------------------#
if debug: print tr+'we create paths if necessary'
#---------------------------------------------------------------#

pathlocal='/var/spool/news/local/'

pathlocalexists=os.path.isdir(pathlocal)
if pathlocalexists!=1:
	os.mkdir(pathlocal)

if debug :print 'pathlocalexists=',pathlocalexists

pathgroup='/var/spool/news/local/'+mailbox+'/'

if debug: print 'pathgroup=',pathgroup

pathgroupexists=os.path.isdir(pathgroup)
if pathgroupexists!=1:
	os.mkdir(pathgroup)

if debug: print 'pathgroupexists=',pathgroupexists

#---------------------------------------------------------------#
if debug: print tr+'we take the initial number in group.info'
#---------------------------------------------------------------#

s=readfile(pathnews+'leaf.node/groupinfo')

target='local.'+mailbox+' '

groupingroupinfo=string.find(s,target)

if groupingroupinfo==-1:
	initial=1
else:
	pattern='local\.'+mailbox+' ([0-9]+) '
	
	match=re.search(pattern,s)
	if match==None:
		print "cannot find the initial number in group.info !"
		sys.exit(2)
	else:
		initial=int(string.strip(match.group(1)))+1

if debug: print 'initial=',initial

#---------------------------------------------------------------#
if debug: print tr+'we cut the mailbox'
#---------------------------------------------------------------#

s=readfile(mailbox)

i=initial
target="\nFrom "
ltarget=len(target)

begining=string.find(s,'From ')

gojohnnygo=begining

while gojohnnygo!=-1:

	end=string.find(s,target,begining+ltarget)
	gojohnnygo=end

	select=s[begining:end]
	
	# to eventually avoid problems with the first 2 \n
	if select[0]=='\n':
		select=select[1:]

	namefile=repr(i)		

	savefile(select,namefile)

	begining=end
	i=i+1

nbfiles=i-initial

if debug: print 'nbfiles=',nbfiles

#---------------------------------------------------------------#
if debug: print tr+'we take the headers we need'
#---------------------------------------------------------------#

listheaders=[]

for i in range(initial,nbfiles+initial):
			
	file=repr(i)
	s=readfile(file)
	
	if debug: print 'file=',file

	target='\n\n'
	cut=string.find(s,target)
	headers=s[:cut]
	
	pattern='\nSubject:(.+)'
	match=re.search(pattern,headers)
	if match==None:
		Subject=''
	else:
		Subject=string.strip(match.group(1))
		Subject=string.replace(Subject,'\t',' ')
	
	if debug: print 'Subject=',Subject

	pattern='\nFrom:(.+)'
	match=re.search(pattern,headers)
	if match==None:
		From=''
	else:
		From=string.strip(match.group(1))
		From=string.replace(From,'\t',' ')

	if debug: print 'From=',From
	
	pattern='\nDate:(.+)'
	match=re.search(pattern,headers)
	if match==None:
		Date=''
	else:
		Date=string.strip(match.group(1))
		Date=string.replace(Date,'\t',' ')
	
	if debug: print 'Date=',Date
	
	pattern='\nMessage-I(D|d): *(<.+>)'
	match=re.search(pattern,headers)
	if match==None:
		# we absolutly need one for the hard links
		if debug: print 'no Message-ID ! We construct one'
		a1=whrandom.randint(0,99999)
		a2=whrandom.randint(0,99999)
		a3=whrandom.randint(0,99999)
		a4=whrandom.randint(0,99999)
		MessageID='<'+repr(a1)+'.'+repr(a2)+'@local.'+repr(a3)+ \
		'.'+repr(a4)+'>'
	else:
		MessageID=string.strip(match.group(2))
		# some M-Id are not conforms and contains \t
		MessageID=string.replace(MessageID,'\t','-')

	# to avoid problems with messageid that already exists
	# note that we only check messageids in the spool
	# it would be nice to check also the current mailbox...
	messageidexists=os.path.isfile(pathids+MessageID)
	if messageidexists:
		if debug: print 'Message-ID already exits ! We construct another one'
		a1=whrandom.randint(0,99999)
		a2=whrandom.randint(0,99999)
		MessageID=MessageID[:-1]+'.local.'+repr(a1)+'.'+repr(a2)+'>'
		
	if debug: print 'MessageID=',MessageID
	
	pattern='\nReferences:( *<.+>(\n\t)*)+'
	match=re.search(pattern,headers)
	if match==None:
		# we try to use the In-Reply-To field if it is asked
		pattern='\nIn-Reply-To: *(<.+>)'
		match=re.search(pattern,headers)
		if UseInReplyTo==0 or match==None:
			References=''
		else:			
			References=string.strip(match.group(1))
			if debug: print 'no References but use a In-Reply-To'
	else:
		References=string.strip(match.group(1))
	References=string.replace(References,'\n\t',' ')
	References=string.replace(References,'\t',' ')
		
	if debug: print 'References=',References

	listheaders.append((Subject,From,Date,MessageID,References))

	if debug: print

#---------------------------------------------------------------#
if debug: print tr+'we transform and add necessary headers'
#---------------------------------------------------------------#

if debug: print 'nbfiles=',nbfiles
if debug: print 'initial=',initial

hostname=os.uname()[1]

for i in range(0,nbfiles):
	
	if debug: print 'i=',i
	
	file=repr(initial+i)
	s=readfile(file)

	target='\n\n'
	cut=string.find(s,target)
	headers=s[:cut]
	body=s[cut:]
	
	# to avoid problem with the real From:
	headers=string.replace(headers,"From ","X-From: ",1)
	
	# if there is no References field in headers and if we created one
	# with the In-Reply-To, we use it 
	IsReferences=string.find(headers,'\nReferences:')
	References=listheaders[i][4]
	if IsReferences==-1 and References!='':
		headers=headers+'\nReferences: '+References

	number=repr(initial+i)
	Xref='Xref: '+hostname+' local.'+mailbox+':'+number
	Newsgroups='Newsgroups: local.'+mailbox
	FollowupTo='Followup-To: poster'
	
	headers=headers+'\n'+Newsgroups+'\n'+FollowupTo+'\n'+Xref
	
	s=headers+body
	
	if DelHtml==1:
		s=delhtml(s)
	
	savefile(s,file)

#---------------------------------------------------------------#
if debug: print tr+"we create the new .overview"
#---------------------------------------------------------------#

newoverview=""

for i in range(0,nbfiles):

	if debug: print 'i=',i
	
	file=repr(initial+i)
	s=readfile(file)
	byte=repr(len(s))
	lines=repr(string.count(s,"\n")+1)
	
	number=repr(i+initial)
	Xref="Xref:  local:"+number
	Subject=listheaders[i][0]
	From=listheaders[i][1]
	Date=listheaders[i][2]
	MessageID=listheaders[i][3]
	References=listheaders[i][4]
	
	ligne=number+"\t"+Subject+"\t"+From+"\t"+Date+"\t"\
	+MessageID+"\t"+References+"\t"+byte+"\t"+lines+\
	"\t"+Xref+"\n"

	newoverview=newoverview+ligne

pathoverview=pathgroup+'.overview'
overviewxexists=os.path.isfile(pathoverview)
if debug: print "overviewxexists=",overviewxexists

if overviewxexists!=1:
	overview=newoverview
else:
	overview=readfile(pathoverview)
	overview=overview+newoverview

savefile(overview,'.overview')

#---------------------------------------------------------------#
if debug: print tr+'we move the files in news/local'
#---------------------------------------------------------------#

#args=''
#for i in range(0,nbfiles):
#	args=args+' '+repr(i+initial)
#command='mv -f '+args+' '+pathgroup
#if debug: print command
#os.system(command)
#XXX
for i in range(0,nbfiles):
	file=repr(i+initial)
	if debug: print 'mv '+file+' '+pathgroup
	shutil.copy(file,pathgroup)
	os.remove(file)

#command='mv -f .overview '+pathgroup
#if debug: print command
#os.system(command)
#XXX
if debug: print 'mv .overview '+pathgroup
shutil.copy('.overview',pathgroup)
os.remove('.overview')

#---------------------------------------------------------------#
if debug: print tr+'we create the hard links in message.id/'
#---------------------------------------------------------------#

for i in range(0,nbfiles):

	MessageID=listheaders[i][3]
	file=repr(i+initial)
	
#	command='ln -f '+pathgroup+file+" '"+pathids+MessageID+"'"
#	if debug: print command
#	os.system(command)
#XXX

	pattern='[^0-9a-zA-Z\.\$@\<\>\+\-]+'
	match=re.search(pattern,MessageID)
	if match==None:
		if debug: print '(hard) link '+pathgroup+file+' '+pathids+MessageID
		os.link(pathgroup+file,pathids+MessageID)
	else:
		if debug: print 'Bad format of MessageID for the hard link !'
		if debug: print 'MessageID='MessageID
	
#---------------------------------------------------------------#
if debug: print tr+'we update group.info'
#---------------------------------------------------------------#

s=readfile(pathnews+'leaf.node/groupinfo')

number=nbfiles+initial-1

if debug: print 'new number=',number

if groupingroupinfo==-1:
	if debug: print 'we create the line in group.info'
	s='local.'+mailbox+' '+repr(number)+' 1 0 -x-\n'+s
	
else:
	if debug: print 'we replace the ligne in group.info'
	pattern='local\.'+mailbox+' ([0-9]+) '
	repl='local.'+mailbox+' '+repr(number)+' '
	s=re.sub(pattern,repl,s)

savefile(s,pathnews+'leaf.node/groupinfo')

#---------------------------------------------------------------#
if debug: print tr+'we remove the original mailbox'
#---------------------------------------------------------------#

#command='rm -f '+pathcompletmailbox
#if debug: print command
#os.system(command)
#XXX
if debug: print 'remove '+pathcompletmailbox
os.remove(pathcompletmailbox)

#---------------------------------------------------------------#
if debug: print tr+"ok."
#---------------------------------------------------------------#

